@@ -245,7 +245,7 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 		ChnSettings[chn].nPan = (((chn & 3) == 1) || ((chn & 3) == 2)) ? 64 : 192;
 	}
 
-	uint8 sampleNameLength[64];
+	uint8 sampleNameLength[64] = {};
 	for(SAMPLEINDEX smp = 1; smp <= m_nSamples; smp++)
 	{
 		Samples[smp].Initialize(MOD_TYPE_MOD);
@@ -604,7 +604,7 @@ bool CSoundFile::ReadDSym(FileReader &file, ModLoadingFlags loadFlags)
 	m_modFormat.formatName = MPT_UFORMAT("Digital Symphony v{}")(fileHeader.version);
 	m_modFormat.type = U_("dsym");  // RISC OS doesn't use file extensions but this is a common abbreviation used for this tracker
 	m_modFormat.madeWithTracker = U_("Digital Symphony");
-	m_modFormat.charset = mpt::Charset::ISO8859_1;  // Close enough (RISC OS uses slightly customized variant)
+	m_modFormat.charset = mpt::Charset::RISC_OS;
 
 	return true;
 }
