@@ -12,6 +12,7 @@
 #include "Sndfile.h"
 #include "ModSample.h"
 #include "modsmp_ctrl.h"
+#include "mpt/base/numbers.hpp"
 
 #include <cmath>
 
@@ -52,7 +53,7 @@ void ModSample::Convert(MODTYPE fromType, MODTYPE toType)
 		RelativeTone = 0;
 	}
 
-	// No global volume sustain loops for MOD/S3M/XM
+	// No global volume / sustain loops for MOD/S3M/XM
 	if(toType & (MOD_TYPE_MOD | MOD_TYPE_XM | MOD_TYPE_S3M))
 	{
 		nGlobalVol = 64;
@@ -147,7 +148,7 @@ void ModSample::Initialize(MODTYPE type)
 	rootNote = 0;
 	filename = "";
 
-	SetDefaultCuePoints();
+	RemoveAllCuePoints();
 }
 
 
@@ -166,6 +167,27 @@ uint32 ModSample::GetSampleRate(const MODTYPE type) const
 }
 
 
+// Copies sample data from another sample slot and ensures that the 16-bit/stereo flags are set accordingly.
+bool ModSample::CopyWaveform(const ModSample &smpFrom)
+{
+	if(!smpFrom.HasSampleData())
+		return false;
+	// If we duplicate a sample slot, avoid deleting the sample we just copy from
+	if(smpFrom.sampleb() == sampleb())
+		pData.pSample = nullptr;
+	LimitMax(nLength, smpFrom.nLength);
+	uFlags.set(CHN_16BIT, smpFrom.uFlags[CHN_16BIT]);
+	uFlags.set(CHN_STEREO, smpFrom.uFlags[CHN_STEREO]);
+	if(AllocateSample())
+	{
+		memcpy(sampleb(), smpFrom.sampleb(), GetSampleSizeInBytes());
+		return true;
+	}
+	return false;
+
+}
+
+
 // Allocate sample based on a ModSample's properties.
 // Returns number of bytes allocated, 0 on failure.
 size_t ModSample::AllocateSample()
@@ -194,7 +216,7 @@ void *ModSample::AllocateSample(SmpLength numFrames, size_t bytesPerSample)
 		if(p != nullptr)
 		{
 			memset(p, 0, allocSize);
-			return p + (InterpolationMaxLookahead * MaxSamplingPointSize);
+			return p + (InterpolationLookaheadBufferSize * MaxSamplingPointSize);
 		}
 	}
 	return nullptr;
@@ -212,7 +234,7 @@ size_t ModSample::GetRealSampleBufferSize(SmpLength numSamples, size_t bytesPerS
 	// * 4x InterpolationMaxLookahead for the sustain loop (same as the two points above)
 
 	const SmpLength maxSize = Util::MaxValueOfType(numSamples);
-	const SmpLength lookaheadBufferSize = (MaxSamplingPointSize + 1 + 4 + 4) * InterpolationMaxLookahead;
+	const SmpLength lookaheadBufferSize = (MaxSamplingPointSize + 1 + 4 + 4) * InterpolationLookaheadBufferSize;
 
 	if(numSamples == 0 || numSamples > MAX_SAMPLE_LENGTH || lookaheadBufferSize > maxSize - numSamples)
 	{
@@ -240,7 +262,7 @@ void ModSample::FreeSample(void *samplePtr)
 {
 	if(samplePtr)
 	{
-		delete[](((char *)samplePtr) - (InterpolationMaxLookahead * MaxSamplingPointSize));
+		delete[](((char *)samplePtr) - (InterpolationLookaheadBufferSize * MaxSamplingPointSize));
 	}
 }
 
@@ -311,8 +333,8 @@ public:
 	void CopyLoop(bool direction) const
 	{
 		// Direction: true = start reading and writing forward, false = start reading and writing backward (write direction never changes)
-		const int numSamples = 2 * InterpolationMaxLookahead + (direction ? 1 : 0);  // Loop point is included in forward loop expansion
-		T *dest = target + numChannels * (2 * InterpolationMaxLookahead - 1);        // Write buffer offset
+		const int numSamples = 2 * InterpolationLookaheadBufferSize + (direction ? 1 : 0);  // Loop point is included in forward loop expansion
+		T *dest = target + numChannels * (2 * InterpolationLookaheadBufferSize - 1);        // Write buffer offset
 		SmpLength readPosition = loopEnd - 1;
 		const int writeIncrement = direction ? 1 : -1;
 		int readIncrement = writeIncrement;
@@ -363,7 +385,7 @@ template <typename T>
 void PrecomputeLoopsImpl(ModSample &smp, const CSoundFile &sndFile)
 {
 	const int numChannels = smp.GetNumChannels();
-	const int copySamples = numChannels * InterpolationMaxLookahead;
+	const int copySamples = numChannels * InterpolationLookaheadBufferSize;
 
 	T *sampleData = static_cast<T *>(smp.samplev());
 	T *afterSampleStart = sampleData + smp.nLength * numChannels;
@@ -372,7 +394,7 @@ void PrecomputeLoopsImpl(ModSample &smp, const CSoundFile &sndFile)
 
 	// Hold sample on the same level as the last sampling point at the end to prevent extra pops with interpolation.
 	// Do the same at the sample start, too.
-	for(int i = 0; i < (int)InterpolationMaxLookahead; i++)
+	for(int i = 0; i < (int)InterpolationLookaheadBufferSize; i++)
 	{
 		for(int c = 0; c < numChannels; c++)
 		{
@@ -457,13 +479,13 @@ void ModSample::TransposeToFrequency()
 }
 
 
-// Return a pair of {tranpose, finetune}
+// Return a pair of {transpose, finetune}
 std::pair<int8, int8> ModSample::FrequencyToTranspose(uint32 freq)
 {
 	if(!freq)
 		return {};
 
-	const auto f2t = mpt::saturate_round<int32>(std::log(freq * (1.0 / 8363.0)) * (12.0 * 128.0 * (1.0 / M_LN2)));
+	const auto f2t = mpt::saturate_round<int32>(std::log(freq * (1.0 / 8363.0)) * (12.0 * 128.0 * (1.0 / mpt::numbers::ln2)));
 	const auto fine = std::div(Clamp(f2t, -16384, 16383), int32(128));
 	return {static_cast<int8>(fine.quot), static_cast<int8>(fine.rem)};
 }
@@ -482,16 +504,29 @@ void ModSample::Transpose(double amount)
 }
 
 
+// Check if the sample has any valid cue points
+bool ModSample::HasAnyCuePoints() const
+{
+	if(uFlags[CHN_ADLIB])
+		return false;
+	for(auto pt : cues)
+	{
+		if(pt < nLength)
+			return true;
+	}
+	return false;
+}
+
+
 // Check if the sample's cue points are the default cue point set.
 bool ModSample::HasCustomCuePoints() const
 {
-	if(!uFlags[CHN_ADLIB])
+	if(uFlags[CHN_ADLIB])
+		return false;
+	for(SmpLength i = 0; i < std::size(cues); i++)
 	{
-		for(SmpLength i = 0; i < CountOf(cues); i++)
-		{
-			if(cues[i] != (i + 1) << 11)
-				return true;
-		}
+		if(cues[i] != (i + 1) << 11)
+			return true;
 	}
 	return false;
 }
@@ -506,6 +541,24 @@ void ModSample::SetDefaultCuePoints()
 	}
 }
 
+
+void ModSample::Set16BitCuePoints()
+{
+	// Cue points that are useful for extending regular offset command
+	for(int i = 0; i < 9; i++)
+	{
+		cues[i] = (i + 1) << 16;
+	}
+}
+
+
+void ModSample::RemoveAllCuePoints()
+{
+	if(!uFlags[CHN_ADLIB])
+		cues.fill(MAX_SAMPLE_LENGTH);
+}
+
+
 void ModSample::SetAdlib(bool enable, OPLPatch patch)
 {
 	if(!enable && uFlags[CHN_ADLIB])
